package com.emmetgray.wrpn;

// The main calculator stack
public class CStack
{
    public static final int STACK_SIZE = 4;

    private Register[] pfvReg  = new Register[STACK_SIZE];

    // The default constructor
    public CStack()
    {
        for (int i = 0; i < STACK_SIZE; i++)
        {
            pfvReg[i] = new Register();
        }
    }

    // Build a new stack with the given Big Integer size
    public CStack(int size, BigInt.ArithMode mode)
    {
        for (int i = 0; i < STACK_SIZE; i++)
        {
            pfvReg[i] = new Register(size, mode);
        }
    }

    // Peek at the value at the "bottom" of the stack
    public Register peek()
    {
        return pfvReg[0];
    }

    // Get a Register value off the stack
    public Register pop()
    {
        Register prev;

        prev = pfvReg[0];
        // drop the stack
        // Note: the top of the stack is preserved
        for (int i = 0; i < STACK_SIZE-1; i++)
        {
            pfvReg[i] = pfvReg[i + 1];
        }
        // Make "deep copy" so 2 and 3 aren't pointing to the same
        // register (since register is a reference type).
        pfvReg[3] = pfvReg[3].copy();
        return prev;
    }

    // Push a value onto the stack
    public void push(Register val)
    {
        // push a value on the stack
        // Note: the top of the stack is lost
        for (int i = 2; i >= 0; i--)
        {
            pfvReg[i + 1] = pfvReg[i];
        }
        pfvReg[0] = val;
    }

    // The X register
    public Register getX()
    {
        return pfvReg[0];
    }

    public void setX(Register x)
    {
        pfvReg[0] = x;
    }

    // The Y register
    public Register getY()
    {
        return pfvReg[1];
    }

    public void setY(Register y)
    {
        pfvReg[1] = y;
    }

    // The Z register
    public Register getZ()
    {
        return pfvReg[2];
    }

    public void setZ(Register z)
    {
        pfvReg[2] = z;
    }

    // The T register
    public Register getT()
    {
        return pfvReg[3];
    }

    public void setT(Register t)
    {
        pfvReg[3] = t;
    }

    // Copy the stack to an ordinary array
    public Register[] toArray()
    {
        Register[] ans = new Register[STACK_SIZE];
        for (int i = 0; i < STACK_SIZE; i++)
        {
            ans[3 - i] = pfvReg[i];
        }
        return ans;
    }
}