package com.emmetgray.wrpn;

// A "Return Packet" that contains all of the information
// required by the GUI
public class DisplayPacket
{
    private final int MIN_SLEEP = 0;
    private final int MAX_SLEEP = 5000;

    private String pfvDisplayText;
    private String pfvAlternateText;
    private boolean pfvF_Annunciator;
    private boolean pfvG_Annunciator;
    private boolean pfvCarry_Annunciator;
    private boolean pfvOverflow_Annunciator;
    private boolean pfvPrgm_Annunciator;
    private int pfvDelay;
    private boolean pfvMenuNeedsUpdating;
    private boolean pfvBeep;
    private StartType pfvStart;

    // Which type of program are we starting
    public enum StartType
    {
        None,
        RunProgram,
        RunLine
    }

    // Default constructor
    public DisplayPacket()
    {
        pfvDisplayText = "";
        pfvAlternateText = "";
        pfvF_Annunciator = false;
        pfvG_Annunciator = false;
        pfvCarry_Annunciator = false;
        pfvOverflow_Annunciator = false;
        pfvPrgm_Annunciator = false;
        pfvDelay = 0;
        pfvMenuNeedsUpdating = false;
        pfvBeep = false;
        pfvStart = StartType.None;
    }

    // The main text for the display
    public String getDisplayText()
    {
        return pfvDisplayText;
    }

    public void setDisplayText(String dt)
    {
        if (dt != null)
        {
            pfvDisplayText = dt;
        }
    }

    // The alternate text for the display (typically an error message or pause)
    public String getAlternateText()
    {
        return pfvAlternateText;
    }

    public void setAlternateText(String at)
    {
        if (at != null)
        {
            pfvAlternateText = at;
        }
    }

    // Should the F Annunciator be "lit"
    public boolean getF_Annunciator()
    {
        return pfvF_Annunciator;
    }

    public void setF_Annunciator(boolean f)
    {
        pfvF_Annunciator = f;
    }

    // Should the G Annunciator be "lit"
    public boolean getG_Annunciator()
    {
        return pfvG_Annunciator;
    }

    public void setG_Annunciator(boolean g)
    {
        pfvG_Annunciator = g;
    }

    // Should the Carry Annunciator be "lit"
    public boolean getCarry_Annunciator()
    {
        return pfvCarry_Annunciator;
    }

    public void setCarry_Annunciator(boolean c)
    {
        pfvCarry_Annunciator = c;
    }

    // Should the Overflow Annunciator be "lit"
    public boolean getOverflow_Annunciator()
    {
        return pfvOverflow_Annunciator;
    }

    public void setOverflow_Annunciator(boolean o)
    {
        pfvOverflow_Annunciator = o;
    }

    // Should the Program Annunciator be "lit"
    public boolean getPrgm_Annunciator()
    {
        return pfvPrgm_Annunciator;
    }

    public void setPrgm_Annunciator(boolean p)
    {
        pfvPrgm_Annunciator = p;
    }

    // The delay (in milliseconds) for the paused display
    public int getDelay()
    {
        return pfvDelay;
    }

    public void setDelay(int d)
    {
        if (d >= MIN_SLEEP && d <= MAX_SLEEP)
        {
            pfvDelay = d;
        }
    }

    // Did the calculator engine indicate that the menu needs to be updated?
    public boolean getMenuNeedsUpdating()
    {
        return pfvMenuNeedsUpdating;
    }

    public void setMenuNeedsUpdating(boolean mnu)
    {
        pfvMenuNeedsUpdating = mnu;
    }

    // Did the calculator engine ask for a beep?
    public boolean getBeep()
    {
        return pfvBeep;
    }

    public void setBeep(boolean b)
    {
        pfvBeep = b;
    }

    // Did the calculator engine indicate that a program should start?
    public StartType getStart()
    {
        return pfvStart;
    }

    public void setStart(StartType st)
    {
        pfvStart = st;
    }
}