package com.emmetgray.wrpn;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Paint;
import android.util.AttributeSet;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

/*
 * Adopted from ilharco at https://stackoverflow.com/questions/5033012/auto-scale-textview-text-to-fit-within-bounds
 */
public class DynamicText extends androidx.appcompat.widget.AppCompatEditText
{
    private final int MIN_SIZE = 5;
    private final int MAX_SIZE = 100;

    private int pfvMinFontSize = 5;        // minimum font size
    private int pfvMaxFontSize = 100;        // maximum font size
    private boolean pfvAutoScale = true;    // enabled the autoscaling feature

    private int widthLimitPixels= 0;
    private float calcMax = 0;              // the calculated max font based on height
    private Paint paint = null;

    public DynamicText(@NonNull Context context)
    {
        super(context);
    }

    public DynamicText(@NonNull Context context, @Nullable AttributeSet attrs)
    {
        super(context, attrs);
        // load the parameters from the XML design file
        TypedArray a = getContext().obtainStyledAttributes(attrs, R.styleable.DynamicText);
        // wash these thru the setters
        setMinFontSize(a.getInt(R.styleable.DynamicText_minFontSize, 5));
        setMaxFontSize(a.getInt(R.styleable.DynamicText_maxFontSize, 100));
        setAutoScale(a.getBoolean(R.styleable.DynamicText_autoScale, true));
        a.recycle();
    }

    public DynamicText(@NonNull Context context, AttributeSet attrs, int defStyleAttr)
    {
        super(context, attrs, defStyleAttr);
        TypedArray a = getContext().obtainStyledAttributes(attrs, R.styleable.DynamicText);
        setMinFontSize(a.getInt(R.styleable.DynamicText_minFontSize, 5));
        setMaxFontSize(a.getInt(R.styleable.DynamicText_maxFontSize, 100));
        setAutoScale(a.getBoolean(R.styleable.DynamicText_autoScale, true));
        a.recycle();
    }

    public int getMinFontSize()
    {
        return pfvMinFontSize;
    }

    public void setMinFontSize(int val)
    {
        if (val >= MIN_SIZE)
        {
            pfvMinFontSize = val;
        }
    }

    public int getMaxFontSize()
    {
        return pfvMaxFontSize;
    }

    // TODO: Check to make sure max is larger than min
    public void setMaxFontSize(int val)
    {
        if (val <= MAX_SIZE)
        {
            pfvMaxFontSize = val;
        }
    }

    public boolean getAutoScale()
    {
        return pfvAutoScale;
    }

    public void setAutoScale(boolean val)
    {
        pfvAutoScale = val;
    }

    @Override
    protected void onTextChanged(CharSequence text, int start, int lengthBefore, int lengthAfter)
    {
        super.onTextChanged(text, start, lengthBefore, lengthAfter);

        // if this event fires "too early", it will have a width of zero
        if (pfvAutoScale && this.getWidth() > 0)
        {
            resizeFont();
        }
    }

    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b)
    {
        super.onLayout(changed, l, t, r, b);
        // for things like the device rotation
        if (pfvAutoScale)
        {
            // do some one-time calculations when the layout is done
            widthLimitPixels = this.getWidth() - this.getPaddingRight() - this.getPaddingLeft();
            calcMax = pixelsToSp((this.getHeight() - this.getPaddingTop() - this.getPaddingBottom()) / 1.5f);
            paint = new Paint();

            resizeFont();
        }
    }

    /*
     * Do the math to pick the optimal font size.
     */
    private void resizeFont()
    {
         float fontSize = pfvMinFontSize;

        String viewText = this.getText().toString();
        float widthPixels = 0;

        // a quick sanity check
        if (viewText.isEmpty())
        {
            return;
        }

        while (widthPixels < widthLimitPixels && fontSize <= pfvMaxFontSize && fontSize <= calcMax)
        {
            ++fontSize;
            paint.setTextSize(spToPixels(fontSize));
            widthPixels = paint.measureText(viewText);
        }
        --fontSize;

        this.setTextSize(fontSize);
    }

    private float pixelsToSp(float px)
    {
        float scaledDensity = getResources().getDisplayMetrics().scaledDensity;
        return px / scaledDensity;
    }

    private float spToPixels(float sp)
    {
        float scaledDensity = getResources().getDisplayMetrics().scaledDensity;
        return sp * scaledDensity;
    }
}