package com.emmetgray.wrpn;

import android.app.AlertDialog;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.os.Bundle;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Locale;

public class fmConfigure extends AppCompatActivity
{
    private final int MIN_PRGMMEM = 302;    // rather arbitrary
    private final int MAX_PRGMMEM = 1000;   // only support 3 digits
    private final int MIN_SLEEP = 0;
    private final int MAX_SLEEP = 5000;
    private final int MIN_REGS = 32;
    private final int MAX_REGS = 406;
    private final int MIN_FONTSIZE = 0;
    private final int MAX_FONTSIZE = 50;

    private EditText etPrgmMem;
    private CheckBox cbSyncConv;
    private EditText etSleep;
    private EditText etNumRegs;
    private Spinner spOrientation;
    private EditText etFontSize;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.configure);

        ActionBar actionBar = getSupportActionBar();
        actionBar.setTitle(R.string.config_title);
        actionBar.setDisplayHomeAsUpEnabled(true);

        // inflate the controls
        etPrgmMem = findViewById(R.id.etPrgmMem);
        cbSyncConv = findViewById(R.id.cbSyncConv);
        etSleep = findViewById(R.id.etSleep);
        etNumRegs = findViewById(R.id.etNumRegs);
        spOrientation = findViewById(R.id.spOrientation);
        etFontSize = findViewById(R.id.etFontSize); // v7.0.0

        // load the spinner with the options
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this,
                R.array.orientation, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spOrientation.setAdapter(adapter);

        // populate widgets with the existing values
        etPrgmMem.setText(CalcState.prop.getProperty("PrgmMemoryLines"));
        cbSyncConv.setChecked(Boolean.parseBoolean(CalcState.prop.getProperty("SyncConversions")));
        etSleep.setText(CalcState.prop.getProperty("SleepDelay"));
        etNumRegs.setText(CalcState.prop.getProperty("NumRegisters"));
        etFontSize.setText(CalcState.prop.getProperty("FontSize"));

        switch(CalcState.prop.getProperty("Orientation"))
        {
            case "Auto":
                spOrientation.setSelection(0);
                break;
            case "Landscape":
                spOrientation.setSelection(1);
                break;
            case "Portrait":
                spOrientation.setSelection(2);
                break;
        }
    }

    // write the configuration file back
    public void onSave(View v)
    {
        // Do input validation on all numeric input
        int PrgmMem = validateInt(etPrgmMem, "Program Memory Lines", MIN_PRGMMEM, MAX_PRGMMEM);
        if (PrgmMem < MIN_PRGMMEM)
        {
            return;
        }

        int Sleep = validateInt(etSleep, "Sleep Delay", MIN_SLEEP, MAX_SLEEP);
        if (Sleep < MIN_SLEEP)
        {
            return;
        }

        int NumRegs = validateInt(etNumRegs, "Number of Registers", MIN_REGS, MAX_REGS);
        if (NumRegs < MIN_REGS)
        {
            return;
        }

        int FontSize = validateInt(etFontSize, "Font Size", MIN_FONTSIZE, MAX_FONTSIZE);
        if (FontSize < MIN_FONTSIZE)
        {
            return;
        }

        // boolean and spinner controls don't need input validation
        CalcState.prop.setProperty("PrgmMemoryLines", Integer.toString(PrgmMem));
        CalcState.prop.setProperty("SyncConversions", Boolean.toString(cbSyncConv.isChecked()));
        CalcState.prop.setProperty("SleepDelay", Integer.toString(Sleep));
        CalcState.prop.setProperty("NumRegisters", Integer.toString(NumRegs));
        CalcState.prop.setProperty("Orientation", spOrientation.getSelectedItem().toString());
        CalcState.prop.setProperty("FontSize", Integer.toString(FontSize));

        File config = new File(getExternalFilesDir(null), "wrpn.config");
        try
        {
            // save the changes
            CalcState.prop.storeToXML(new FileOutputStream(config), null);
            Toast.makeText(this, getString(R.string.config_save_success), Toast.LENGTH_LONG).show();
        }
        catch (Exception ex)
        {
            final AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle(getString(R.string.config_title));
            builder.setMessage(getString(R.string.config_save_failure) + ex.toString());
            builder.setIcon(android.R.drawable.ic_dialog_alert);
            builder.setPositiveButton(getString(R.string.button_ok), null);
            builder.show();
        }
    }

    // let's blow this pop stand!
    public void onClose(View v)
    {
        finish();
    }

    // validate an integer field
    private int validateInt(EditText et, String name, int minValue, int maxValue)
    {
        int answer = Integer.MIN_VALUE;
        String buffer = et.getText().toString();

        // case 1 - the field is blank
        if (buffer.isEmpty())
        {
            et.setError(name + " can not be blank");
            et.requestFocus();
            beep();
        }
        else
        {
            try
            {
                int temp = Integer.parseInt(buffer);

                // case 3 - is it inside a valid range
                if (temp < minValue || temp > maxValue)
                {
                    et.setError(String.format(Locale.US, "%s must be between %d and %d", name, minValue, maxValue));
                    et.requestFocus();
                    beep();
                }
                else
                {
                    answer = temp;
                }
            }
            catch (Exception ex)
            {
                // case 2 - does it convert
                et.setError("Could not convert input into a valid " + name);
                et.requestFocus();
                beep();
            }
        }
        return answer;
    }

    // make a noise
    private void beep()
    {
        ToneGenerator toneGenerator = new ToneGenerator(AudioManager.STREAM_SYSTEM, ToneGenerator.MAX_VOLUME);
        toneGenerator.startTone(ToneGenerator.TONE_PROP_BEEP);
    }
}

