package com.emmetgray.wrpn;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;

import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Locale;

public class fmMain extends AppCompatActivity
{
    private DynamicText etDisplay;
    private TextView tvFKey;
    private TextView tvGKey;
    private TextView tvCarry;
    private TextView tvOverflow;
    private TextView tvPrgm;

    private CalcState cs;
    private Calculator c;
    private boolean doRestore = false;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        // inflate the controls that we'll need to talk to
        etDisplay = findViewById(R.id.etDisplay);
        tvFKey = findViewById(R.id.tvFKey);
        tvGKey = findViewById(R.id.tvGKey);
        tvCarry = findViewById(R.id.tvCarry);
        tvOverflow = findViewById(R.id.tvOverflow);
        tvPrgm = findViewById(R.id.tvPrgm);

        // alright, let's go...
        File config = new File(getExternalFilesDir(null), "wrpn.config");
        cs = new CalcState(config.getAbsolutePath());
        c = new Calculator(cs);

        // determine the orientation preference (new feature for v6.0.8)
        switch(CalcState.prop.getProperty("Orientation", "Auto")) {
            case "Landscape":
                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
                break;
            case "Portrait":
                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
                break;
            default:
                setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED);
        }

        // determine the font size. A zero means autoscale
        // v7.0.0 - 6 Jun 21: New AutoScale feature
        int fontSize = Integer.parseInt(CalcState.prop.getProperty("FontSize", "0"));
        if (fontSize != 0)
        {
            etDisplay.setAutoScale(false);
            etDisplay.setTextSize(fontSize);
        }

        /*
         * This application needs to store state between normal things like device
         * rotation... plus things like returning from an intent.  So, instead of
         * using both Parcelable/Bundle technique *and* SharedPreferences, I opted
         * to go with just SharedPreferences.  Also, the serialization is using
         * XML to be consistent with other languages/platforms of WRPN.
         */
        // persist data between user-generated events
        SharedPreferences prefs = getPreferences(MODE_PRIVATE);
        if (prefs  != null)
        {
            doRestore = prefs.getBoolean("doRestore", false);
            if (doRestore)
            {
                try
                {
                    cs.deserialize(prefs.getString("CalcState", ""));
                }
                catch (Exception ex)
                {
                    if (BuildConfig.DEBUG)
                    {
                        Log.e(CalcState.TAG, "onCreate deserialize: " + ex.toString());
                    }
                }
            }
            else
            {
                // Is the SaveOnExit feature enabled?
                if (cs.getSaveOnExit())
                {
                    loadExternalState();
                }
            }
        }
        // TODO: Support device rotation while in PRGM mode
        /*
         * This flag is used to determine if we should automatically restore state.  It is
         * typically used immediately before launching an intent
         */
        doRestore = false;

        // process a dummy key to refresh the display
        ProcessPacket(c.ProcessKey(-1));
    }

    /*
     * This is the normal place you'd expect to use the Bundle to save state, but
     * instead, I only use SharedPreferences
     */
    @Override
    protected void onSaveInstanceState(Bundle outState)
    {
        // persist data between user-generated events
        SharedPreferences prefs = getPreferences(MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putBoolean("doRestore", doRestore);
        try
        {
            editor.putString("CalcState", cs.serialize());
        }
        catch (Exception ex)
        {
            if (BuildConfig.DEBUG)
            {
                Log.e(CalcState.TAG, "onSaveInstanceState serialize: " + ex.toString());
            }
        }
        editor.apply();

        // Note: I'm not using the Bundle at all!
        super.onSaveInstanceState(outState);
    }

    @Override
    protected void onRestoreInstanceState(Bundle inState)
    {
        // Note: I'm not using the Bundle at all!
        super.onRestoreInstanceState(inState);

        // persist data between user-generated events
        SharedPreferences prefs = getPreferences(MODE_PRIVATE);
        if (prefs  != null)
        {
            doRestore = prefs.getBoolean("doRestore", false);
            try
            {
                cs.deserialize(prefs.getString("CalcState", ""));
            }
            catch (Exception ex)
            {
                if (BuildConfig.DEBUG)
                {
                    Log.e(CalcState.TAG, "onRestoreInstanceState deserialize: " + ex.toString());
                }
            }
        }
        // refresh the screen after a device rotation
        ProcessPacket(c.ProcessKey(-1));
    }

    @Override
    protected void onStop()
    {
        // store External State to the default file
        if (cs.getSaveOnExit()) {
            try
            {
                saveExternalState();
                // give it a few moments to write before we close
                Thread.sleep(100);
            }
            catch (Exception ignore) {}
        }
        super.onStop();
    }

    // Map a few keys to their corresponding buttons
    // TODO: Really?  How many Android devices have a keyboard?
    public boolean onKeyDown(int keyCode, KeyEvent event)
    {
        switch (keyCode)
        {
            case KeyEvent.KEYCODE_0:
                ProcessPacket(c.ProcessKey(Calculator.k.Key0.getVal()));
                break;
            case KeyEvent.KEYCODE_1:
                ProcessPacket(c.ProcessKey(Calculator.k.Key1.getVal()));
                break;
            case KeyEvent.KEYCODE_2:
                ProcessPacket(c.ProcessKey(Calculator.k.Key2.getVal()));
                break;
            case KeyEvent.KEYCODE_3:
                ProcessPacket(c.ProcessKey(Calculator.k.Key3.getVal()));
                break;
            case KeyEvent.KEYCODE_4:
                ProcessPacket(c.ProcessKey(Calculator.k.Key4.getVal()));
                break;
            case KeyEvent.KEYCODE_5:
                ProcessPacket(c.ProcessKey(Calculator.k.Key5.getVal()));
                break;
            case KeyEvent.KEYCODE_6:
                ProcessPacket(c.ProcessKey(Calculator.k.Key6.getVal()));
                break;
            case KeyEvent.KEYCODE_7:
                ProcessPacket(c.ProcessKey(Calculator.k.Key7.getVal()));
                break;
            case KeyEvent.KEYCODE_8:
                ProcessPacket(c.ProcessKey(Calculator.k.Key8.getVal()));
                break;
            case KeyEvent.KEYCODE_9:
                ProcessPacket(c.ProcessKey(Calculator.k.Key9.getVal()));
                break;
            case KeyEvent.KEYCODE_A:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyA.getVal()));
                break;
            case KeyEvent.KEYCODE_B:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyB.getVal()));
                break;
            case KeyEvent.KEYCODE_C:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyC.getVal()));
                break;
            case KeyEvent.KEYCODE_D:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyD.getVal()));
                break;
            case KeyEvent.KEYCODE_E:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyE.getVal()));
                break;
            case KeyEvent.KEYCODE_F:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyF.getVal()));
                break;
            case KeyEvent.KEYCODE_PLUS:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyAdd.getVal()));
                break;
            case KeyEvent.KEYCODE_MINUS:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyMin.getVal()));
                break;
            case KeyEvent.KEYCODE_STAR:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyMul.getVal()));
                break;
            case KeyEvent.KEYCODE_SLASH:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyDiv.getVal()));
                break;
            case KeyEvent.KEYCODE_PERIOD:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyDp.getVal()));
                break;
            case KeyEvent.KEYCODE_ENTER:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyEnt.getVal()));
                break;
            case KeyEvent.KEYCODE_DEL:
                ProcessPacket(c.ProcessKey(Calculator.k.KeyBSP.getVal()));
                break;
            default:
                return super.onKeyDown(keyCode, event);
        }
        return true;
    }

    /*
     * This is the common event handler for all of the buttons
     */
    public void onButton(View v)
    {
        int keyCode = Integer.decode(v.getTag().toString());

        // any keystroke will terminate a running program
        cs.setPrgmRunning(false);

        // send the key to the calculator engine
        DisplayPacket pkt = c.ProcessKey(keyCode);

        // show the results in the GUI
        ProcessPacket(pkt);

        // should we start a program?
        if (pkt.getStart() == DisplayPacket.StartType.RunProgram)
        {
            // fire up a background thread to run our program
            Thread t = new Thread(RunProgram);
            t.start();
        }
        else if (pkt.getStart() == DisplayPacket.StartType.RunLine)
        {
            // just run one line at a time
            RunLine();
        }
    }

    /*
     * We need to update the GUI with the information that's coming back from
     * the calculator engine via the DisplayPacket
     */
    private void ProcessPacket(DisplayPacket pkt)
    {
        // Display the Shifted Annunciators
        if (pkt.getF_Annunciator())
        {
            tvFKey.setText("f");
        }
        else
        {
            tvFKey.setText("");
        }
        if (pkt.getG_Annunciator())
        {
            tvGKey.setText("g");
        }
        else
        {
            tvGKey.setText("");
        }

        // Display the System Flag Annunciators
        if (pkt.getCarry_Annunciator())
        {
            tvCarry.setText("C");
        }
        else
        {
            tvCarry.setText("");
        }
        if (pkt.getOverflow_Annunciator())
        {
            tvOverflow.setText("G");
        }
        else
        {
            tvOverflow.setText("");
        }

        // Display the Program Annunciator
        if (pkt.getPrgm_Annunciator())
        {
            tvPrgm.setText("PRGM");
        }
        else
        {
            tvPrgm.setText("");
        }

        // Did the engine ask for a beep?
        if (!cs.getPrgmRunning() && pkt.getBeep())
        {
            // based upon crash reports
            beep();
        }

        /*
         * Gravity is unique to this platform, so we switch between left and
         * right depending on the "mode".
         */
        etDisplay.setGravity(Gravity.CENTER_VERTICAL|Gravity.RIGHT);

        // Optionally show an alternate message for a short time
        if (pkt.getDelay() > 0)
        {
            // Alternate text is always right justified
            etDisplay.setText(pkt.getAlternateText());

            Thread t = new Thread()
            {
                @Override
                public void run()
                {
                    // delay before replacing the display text
                    etDisplay.postDelayed(new Runnable()
                    {
                        //@Override
                        public void run()
                        {
                            if (cs.getOpMode() == CalcState.CalcOpMode.Float)
                            {
                                // float mode is the only thing that is left justified
                                etDisplay.setGravity(Gravity.CENTER_VERTICAL|Gravity.LEFT);
                            }
                            etDisplay.setText(pkt.getDisplayText());
                        }
                    }, pkt.getDelay());
                }
            };
            t.start();
        }
        else
        {
            if (pkt.getAlternateText().isEmpty())
            {
                if (cs.getOpMode() == CalcState.CalcOpMode.Float)
                {
                    etDisplay.setGravity(Gravity.CENTER_VERTICAL|Gravity.LEFT);
                }
                etDisplay.setText(pkt.getDisplayText());
            }
            else
            {
                etDisplay.setText(pkt.getAlternateText());
            }
        }
    }

    //region Programming Region
    // change the calculator display in a thread-safe way
    private void SetDisplayText(final String text)
    {
        new Thread(new Runnable() {
            public void run()
            {
                // run this on the GUI thread
                // v6.0.7 - 22 Jul 2015
                runOnUiThread(new Runnable() {
                    public void run()
                    {
                        if (cs.getOpMode() == CalcState.CalcOpMode.Float)
                        {
                            etDisplay.setGravity(Gravity.CENTER_VERTICAL|Gravity.LEFT);
                        }
                        else
                        {
                            etDisplay.setGravity(Gravity.CENTER_VERTICAL|Gravity.RIGHT);
                        }
                        etDisplay.setText(text);
                    }
                });
            }
        }).start();
    }

    // Run a program starting at the current line number
    Runnable RunProgram = new Runnable() {
        public void run()
        {
            while (cs.getPrgmPosition() < cs.getPrgmMemory().size())
            {
                // execute the instructions
                if (RunLine())
                {
                    // Some error occurred
                    break;
                }

                // prepare to process the next line
                cs.incrementPrgrmPosition();

                // stop if somebody pressed a key
                if (!cs.getPrgmRunning())
                {
                    break;
                }
            }
            // v6.0.7 - 22 Jul 2015
            ProcessPacket(c.ProcessKey(-1));
        }
    };

    // Execute instructions at the current program line
    private boolean RunLine()
    {
        String line;
        int k1, k2, k3;
        final DisplayPacket p;

        // A quick sanity check
        // v6.0.2 - 26 Apr 12
        if (cs.getPrgmPosition() < 0 || cs.getPrgmPosition() >= cs.getPrgmMemory().size())
        {
            // if you "step off the edge", then just stop
            return true;
        }

        // get the current line
        line = cs.getPrgmMemory().get(cs.getPrgmPosition());
        //Log.i(CalcState.TAG, "pos=" + (cs.getPrgmPosition() +1) + ", line=" + line);

        /*
         * This is kinda strange... there are 3 different formats for the program line.
         * They can have either 1 argument, 2 arguments, or 3 arguments.
         */
        if (line.startsWith("      "))  // one argument
        {
            try
            {
                k1 = Integer.parseInt(line.substring(6, 8).trim(), 16);
                p = c.ProcessKey(k1);
            }
            catch (Exception ex)
            {
                // I'm anticipating that folks might edit the XML by hand
                // to make minor tweaks to a program. So, we have to be
                // ready for a corrupted file
                alertDialog(R.string.error_prgm_title, getString(R.string.error_prgm_msg) + cs.getPrgmPosition() + "\n" + ex.toString());
                return true;
            }
        }
        else if (line.startsWith("   "))  // two arguments
        {
            try
            {
                // v7.0.0 - 6 Jun 21: Oops, I missed this
                // special case for STO and RCL that optionally use the "." prefix
                if (line.charAt(6) == '.')
                {
                    k1 = Integer.parseInt(line.substring(3, 5).trim(), 16);
                    k2 = Integer.parseInt(line.substring(7, 8).trim(), 16);
                    c.ProcessKey(k1);
                    c.ProcessKey(Calculator.k.KeyDp.getVal());
                    p = c.ProcessKey(k2);
                }
                else
                {
                    k1 = Integer.parseInt(line.substring(3, 5).trim(), 16);
                    k2 = Integer.parseInt(line.substring(6, 8).trim(), 16);
                    c.ProcessKey(k1);
                    p = c.ProcessKey(k2);
                }
            }
            catch (Exception ex)
            {
                alertDialog(R.string.error_prgm_title, getString(R.string.error_prgm_msg) + cs.getPrgmPosition() + "\n" + ex.toString());
                return true;
            }
        }
        else  // Three arguments
        {
            try
            {
                k1 = Integer.parseInt(line.substring(0, 2).trim(), 16);
                k2 = Integer.parseInt(line.substring(3, 5).trim(), 16);
                k3 = Integer.parseInt(line.substring(6, 8).trim(), 16);
                c.ProcessKey(k1);
                c.ProcessKey(k2);
                p = c.ProcessKey(k3);
            }
            catch (Exception ex)
            {
                alertDialog(R.string.error_prgm_title, getString(R.string.error_prgm_msg) + cs.getPrgmPosition() + "\n" + ex.toString());
                return true;
            }
        }

        // We only update the display if there is a pause. No Annunciator
        // flags are updated while a program is running.
        if (p.getDelay() > 0)
        {
            if (p.getAlternateText().isEmpty())
            {
                SetDisplayText(p.getDisplayText());
                try
                {
                    Thread.sleep(p.getDelay());
                }
                catch (Exception ignore) {}
            }
            else
            {
                SetDisplayText(p.getAlternateText());
                try
                {
                    Thread.sleep(p.getDelay());
                }
                catch (Exception ignore) {}
            }
        }
        return false;
    }
    //endregion

    //region Menu Bar Region
    // Create the menu items in the menu bar
    public boolean onCreateOptionsMenu(Menu menu)
    {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.main_menu, menu);
        return true;
    }


    /*
     * load the menu with all of the current settings.  Note: we are not using the
     * DisplayPacket.MenuNeedsUpdating property in this platform
     */

    @Override
    public boolean onPrepareOptionsMenu(Menu menu)
    {
        // Inflate the Mode Menu
        MenuItem mModeFloat, mModeHex, mModeDec, mModeOct, mModeBin, mModeSI;
        mModeFloat = menu.findItem(R.id.mode_float);
        mModeHex = menu.findItem(R.id.mode_hex);
        mModeDec = menu.findItem(R.id.mode_dec);
        mModeOct = menu.findItem(R.id.mode_oct);
        mModeBin = menu.findItem(R.id.mode_bin);
        mModeSI = menu.findItem(R.id.mode_si);

        // start with everything off...
        mModeFloat.setChecked(false);
        mModeHex.setChecked(false);
        mModeDec.setChecked(false);
        mModeBin.setChecked(false);
        mModeOct.setChecked(false);
        mModeSI.setChecked(false);

        switch (cs.getOpMode())
        {
            case Float:
                mModeFloat.setChecked(true);
                if (cs.getFloatPrecision() == Calculator.k.KeyDp.getVal())
                {
                    mModeSI.setChecked(true);
                }
                else
                {
                    mModeSI.setChecked(false);
                }
                break;
            case Hex:
                mModeHex.setChecked(true);
                break;
            case Dec:
                mModeDec.setChecked(true);
                break;
            case Oct:
                mModeOct.setChecked(true);
                break;
            case Bin:
                mModeBin.setChecked(true);
                break;
        }

        // Inflate the Options Menu
        MenuItem mOptionSave, mOption8bit, mOption16bit, mOption32bit, mOption64bit, mOption1sComp, mOption2sComp, mOptionUnsigned;
        mOptionSave = menu.findItem(R.id.opt_save);
        mOption8bit = menu.findItem(R.id.opt_8bit);
        mOption16bit = menu.findItem(R.id.opt_16bit);
        mOption32bit = menu.findItem(R.id.opt_32bit);
        mOption64bit = menu.findItem(R.id.opt_64bit);
        mOption1sComp = menu.findItem(R.id.opt_1s);
        mOption2sComp = menu.findItem(R.id.opt_2s);
        mOptionUnsigned = menu.findItem(R.id.opt_unsigned);

        mOptionSave.setChecked(cs.getSaveOnExit());
        mOption8bit.setChecked(false);
        mOption16bit.setChecked(false);
        mOption32bit.setChecked(false);
        mOption64bit.setChecked(false);

        switch (cs.getWordSize()) {
            case 8:
                mOption8bit.setChecked(true);
                break;
            case 16:
                mOption16bit.setChecked(true);
                break;
            case 32:
                mOption32bit.setChecked(true);
                break;
            case 64:
                mOption64bit.setChecked(true);
                break;
            // unlike a lot of other menu items, this one may not have anything checked
            // because odd-ball word lengths (like 12 or 42) are allowed
        }
        mOption1sComp.setChecked(false);
        mOption2sComp.setChecked(false);
        mOptionUnsigned.setChecked(false);

        switch (cs.getArithMode()) {
            case OnesComp:
                mOption1sComp.setChecked(true);
                break;
            case TwosComp:
                mOption2sComp.setChecked(true);
                break;
            case Unsigned:
                mOptionUnsigned.setChecked(true);
                break;
        }

        // The Flags Menu
        menu.findItem(R.id.flag0).setChecked(cs.getFlag(CalcState.CalcFlag.User0));
        menu.findItem(R.id.flag1).setChecked(cs.getFlag(CalcState.CalcFlag.User1));
        menu.findItem(R.id.flag2).setChecked(cs.getFlag(CalcState.CalcFlag.User2));
        menu.findItem(R.id.flagZeros).setChecked(cs.getFlag(CalcState.CalcFlag.LeadingZero));
        menu.findItem(R.id.flagCarry).setChecked(cs.getFlag(CalcState.CalcFlag.Carry));
        menu.findItem(R.id.flagOverflow).setChecked(cs.getFlag(CalcState.CalcFlag.Overflow));

        return super.onPrepareOptionsMenu(menu);
    }

    // process the user changes
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        int id = item.getItemId();

        if (id == R.id.file_open)
        {
            // create a list of files at the correct location
            File datadir = getExternalFilesDir(null);
            filterBy filter = new filterBy("xml");
            String[] files = datadir.list(filter);
            if (files == null || files.length == 0)
            {
                Toast.makeText(this, getString(R.string.file_open_empty), Toast.LENGTH_LONG).show();
                return true;
            }

            AlertDialog.Builder open_file = new AlertDialog.Builder(this);
            open_file.setTitle(getString(R.string.dialog_open_title));
            open_file.setItems(files, new DialogInterface.OnClickListener()
            {
                public void onClick(DialogInterface dialog, int item)
                {
                    File fullpath = new File(datadir, files[item]);
                    loadExternalState(fullpath);
                    ProcessPacket(c.ProcessKey(-1));
                }
            });
            open_file.show();
            return true;
        }
        else if (id == R.id.file_save)
        {
            saveExternalState();
            return true;
        }
        else if (id == R.id.file_saveas)
        {
            AlertDialog.Builder save_file = new AlertDialog.Builder(this);
            save_file.setTitle(getString(R.string.dialog_save_title));
            save_file.setMessage(getString(R.string.dialog_save_msg));
            // we need a text box for entering the file name
            final EditText input = new EditText(this);
            save_file.setView(input);
            save_file.setPositiveButton(getString(R.string.button_ok), new DialogInterface.OnClickListener()
            {
                public void onClick(DialogInterface dialog, int whichButton)
                {
                    String filename = input.getText().toString();
                    if (!filename.toLowerCase(Locale.US).endsWith(".xml"))
                    {
                        filename += ".xml";
                    }
                    File fullpath = new File(getExternalFilesDir(null), filename);
                    saveExternalState(fullpath);
                }
            });
            save_file.setNegativeButton(getString(R.string.button_cancel), null);
            save_file.show();
            return true;
        }
        else if (id == R.id.file_exit)
        {
            finish();
            return true;
        }
        else if (id == R.id.mode_float)
        {
            cs.setOpMode(CalcState.CalcOpMode.Float);
            cs.setFloatPrecision(3);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.mode_hex)
        {
            cs.setOpMode(CalcState.CalcOpMode.Hex);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.mode_dec)
        {
            cs.setOpMode(CalcState.CalcOpMode.Dec);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.mode_oct)
        {
            cs.setOpMode(CalcState.CalcOpMode.Oct);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.mode_bin)
        {
            cs.setOpMode(CalcState.CalcOpMode.Bin);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.mode_si)
        {
            cs.setOpMode(CalcState.CalcOpMode.Float);
            if (cs.getFloatPrecision() == Calculator.k.KeyDp.getVal())
            {
                cs.setFloatPrecision(3);
            }
            else
            {
                cs.setFloatPrecision(Calculator.k.KeyDp.getVal());
            }
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.opt_clear)
        {
            loadExternalState(this.getResources().openRawResource(R.raw.calcstate));
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.opt_save)
        {
            cs.setSaveOnExit(!cs.getSaveOnExit());
            return true;
        }
        else if (id == R.id.opt_8bit)
        {
            cs.setWordSize(8);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.opt_16bit)
        {
            cs.setWordSize(16);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.opt_32bit)
        {
            cs.setWordSize(32);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.opt_64bit)
        {
            cs.setWordSize(64);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.opt_1s)
        {
            cs.setArithMode(BigInt.ArithMode.OnesComp);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.opt_2s)
        {
            cs.setArithMode(BigInt.ArithMode.TwosComp);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.opt_unsigned)
        {
            cs.setArithMode(BigInt.ArithMode.Unsigned);
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.flag0)
        {
            cs.setFlag(CalcState.CalcFlag.User0, !cs.getFlag(CalcState.CalcFlag.User0));
            return true;
        }
        else if (id == R.id.flag1)
        {
            cs.setFlag(CalcState.CalcFlag.User1, !cs.getFlag(CalcState.CalcFlag.User1));
            return true;
        }
        else if (id == R.id.flag2)
        {
            cs.setFlag(CalcState.CalcFlag.User2, !cs.getFlag(CalcState.CalcFlag.User2));
            return true;
        }
        else if (id == R.id.flagZeros)
        {
            cs.setFlag(CalcState.CalcFlag.LeadingZero, !cs.getFlag(CalcState.CalcFlag.LeadingZero));
            ProcessPacket(c.ProcessKey(-1));
        }
        else if (id == R.id.flagCarry)
        {
            cs.setFlag(CalcState.CalcFlag.Carry, !cs.getFlag(CalcState.CalcFlag.Carry));
            ProcessPacket(c.ProcessKey(-1));
            return true;
        }
        else if (id == R.id.flagOverflow)
        {
            cs.setFlag(CalcState.CalcFlag.Overflow, !cs.getFlag(CalcState.CalcFlag.Overflow));
            ProcessPacket(c.ProcessKey(-1));
        }
        else if (id == R.id.config)
        {
            // this flag is to restore state after the intent
            doRestore = true;
            Intent myConfigIntent = new Intent(this, fmConfigure.class);
            startActivity(myConfigIntent);
            return true;
        }
        else if (id == R.id.privacy)
        {
            // this flag is to restore state after the intent
            doRestore = true;
            Intent myPrivancyIntent = new Intent(this, fmPrivacy.class);
            startActivity(myPrivancyIntent);
            return true;
        }
        else if (id == R.id.help_content)
        {
            doRestore = true;
            Uri uri = Uri.parse(CalcState.prop.getProperty("HelpURL"));
            Intent myContentIntent = new Intent(Intent.ACTION_VIEW, uri);
            startActivity(myContentIntent);
            return true;
        }
        else if (id == R.id.help_about)
        {
            doRestore = true;
            Intent myAboutIntent = new Intent(this, fmAbout.class);
            startActivity(myAboutIntent);
            return true;
        }

        else if (id == R.id.help_backpanel)
        {
            doRestore = true;
            Intent myBackPanelIntent = new Intent(this, fmBackPanel.class);
            startActivity(myBackPanelIntent);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }
    //endregion

    //region External State Region
    /*
     * There's practically no different between the SharedPreferences and this so-called
     * "external" state file.  The only real difference is that we control the name of the file,
     * and can have multiple files (for storing programs)
     */

    // load the external state from the default file
    private void loadExternalState()
    {
        File CalcState = new File(getExternalFilesDir(null), "CalcState.xml");
        if (CalcState.exists())
        {
            loadExternalState(CalcState);
        }
    }

    // load the external state file from the "named" file
    private void loadExternalState(File f)
    {
        try
        {
            FileInputStream fis = new FileInputStream(f);
            loadExternalState(fis);
        }
        catch (Exception ex)
        {
            // TODO: Consider deleting the corrupted file upon error
            alertDialog(R.string.error_title_read, getString(R.string.error_msg_read) + "\n" + ex.toString());
        }
    }

    // load the state file from an input stream.  Typically used to "reset" the state
    private void loadExternalState(InputStream stream)
    {
        BufferedReader sr = null;

        try
        {
            StringBuilder sb = new StringBuilder();
            String line;
            sr = new BufferedReader(new InputStreamReader(stream));

            while ((line = sr.readLine()) != null)
            {
                sb.append(line);
            }
            cs.deserialize(sb.toString());
        }
        catch (Exception ex)
        {
            alertDialog(R.string.error_title_read, getString(R.string.error_msg_read) + "\n" + ex.toString());
        }
        finally
        {
            if (sr != null)
            {
                try
                {
                    sr.close();
                }
                catch (IOException ignore){}
            }
        }
    }

    // save to the external state file
    private void saveExternalState()
    {
        File CalcState = new File(getExternalFilesDir(null), "CalcState.xml");
        saveExternalState(CalcState);
    }

    // save to an external "named" state file
    private void saveExternalState(File f)
    {
        BufferedWriter sw = null;

        try
        {
            sw = new BufferedWriter(new FileWriter(f));
            sw.write(cs.serialize());
            sw.flush();
        }
        catch (Exception ex)
        {
            alertDialog(R.string.error_title_save, getString(R.string.error_msg_save) + "\n" + ex.toString());
        }
        finally
        {
            if (sw != null)
            {
                try
                {
                    sw.flush();
                    sw.close();
                }
                catch (IOException ignore){}
            }
        }
    }
    //endregion

    //region Misc Methods Region
    // filter files by extension
    private static class filterBy implements FilenameFilter
    {
        private String pExtension;

        public filterBy(String extension)
        {
            // TODO: Check to see if the user already supplied the "."
            pExtension = "." + extension;
        }

        public boolean accept(File directory, String filename)
        {
            return filename.endsWith(pExtension);
        }
    }

    // A simple error dialog message
    private void alertDialog(int errTitle, String errMessage)
    {
        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        dialog.setMessage(errMessage);
        dialog.setTitle(errTitle);
        dialog.setIconAttribute(android.R.attr.alertDialogIcon);
        dialog.setPositiveButton(getString(R.string.button_ok), null);

        AlertDialog alertDialog = dialog.create();
        alertDialog.show();
    }

    // make a noise
    private void beep()
    {
        ToneGenerator toneGenerator = new ToneGenerator(AudioManager.STREAM_SYSTEM, ToneGenerator.MAX_VOLUME);
        toneGenerator.startTone(ToneGenerator.TONE_PROP_BEEP);
    }

    //endregion
}