package com.emmetgray.wrpn;

public class Prgm
{
    private CalcState cs;
    private Calculator c;

    //  A,  B,  C,  D,  E,  F, 7,  8,  9, 10
    // 21, 22, 23, 24, 25, 26, 4,  5,  6, 20
    // 31, 32, 33, 35, 35, 36, 1,  2,  3, 30
    // 41, 42, 43, 44, 45,     0, 48, 49, 40
    private final int KeyA = 0xa;
    private final int KeyB = 0xb;
    private final int KeyC = 0xc;
    private final int KeyD = 0xd;
    private final int KeyE = 0xe;
    private final int KeyF = 0xf;
    private final int Key7 = 0x7;
    private final int Key8 = 0x8;
    private final int Key9 = 0x9;
    private final int KeyDiv = 0x10;
    private final int KeyGSB = 0x21;
    private final int KeyGTO = 0x22;
    private final int KeyHEX = 0x23;
    private final int KeyDEC = 0x24;
    private final int KeyOCT = 0x25;
    private final int KeyBIN = 0x26;
    private final int Key4 = 0x4;
    private final int Key5 = 0x5;
    private final int Key6 = 0x6;
    private final int KeyMul = 0x20;
    private final int KeyRS = 0x31;
    private final int KeySST = 0x32;
    private final int KeyRol = 0x33;
    private final int KeyXY = 0x34;
    private final int KeyBSP = 0x35;
    private final int KeyEnt = 0x36;
    private final int Key1 = 0x1;
    private final int Key2 = 0x2;
    private final int Key3 = 0x3;
    private final int KeyMin = 0x30;
    private final int KeyON = 0x41;
    private final int KeyFKey = 0x42;
    private final int KeyGKey = 0x43;
    private final int KeySTO = 0x44;
    private final int KeyRCL = 0x45;
    private final int Key0 = 0x0;
    private final int KeyDp = 0x48;
    private final int KeyCHS = 0x49;
    private final int KeyAdd = 0x40;

    public Prgm(CalcState cs1, Calculator c1)
    {
        cs = cs1;
        c = c1;
    }

    public void simpleProgram()
    {
        DisplayPacket dp;

        dp = c.ProcessKey(-1);
        if (!dp.getPrgm_Annunciator())  // are we already in PGRM mode?
        {
            c.ProcessKey(KeyGKey);
            c.ProcessKey(KeyRS);        // enter PGRM mode
        }

        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyRol);           // clear the program

        c.ProcessKey(1);                                 // 001
        c.ProcessKey(0);        // put a 10 in x         // 002

        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyGTO);          // swap with I           // 003

        c.ProcessKey(5);        // put a 5 in Reg 0     // 004

        c.ProcessKey(KeySTO);
        c.ProcessKey(0);                                // 005

        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyGTO);
        c.ProcessKey(KeyA);             // build label A        // 006

        c.ProcessKey(KeyGSB);
        c.ProcessKey(KeyB);             // call subroutine B    // 007

        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyXY);            // pause                // 008

        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyHEX);           // DSZ                  // 009

        c.ProcessKey(KeyGTO);
        c.ProcessKey(KeyA);             // Goto A               // 010

        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyGSB);           // return               // 011

        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyGTO);
        c.ProcessKey(KeyB);             // build Label B        // 012

        c.ProcessKey(KeyRCL);
        c.ProcessKey(0);        // get what's in Reg 0  // 013

        c.ProcessKey(1);                                // 014

        c.ProcessKey(KeyAdd);           // add one...          // 015

        c.ProcessKey(KeySTO);
        c.ProcessKey(0);        // store it back        // 016

        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyGSB);           // return               // 017

        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyRS);            // exit PGRM mode
    }

    /*
     * This is a non-threaded version of the Runnable in fmMain
     */
    public void run()
    {
        cs.setPrgmRunning(true);

        while (cs.getPrgmPosition() < cs.getPrgmMemory().size())
        {
            // execute the instructions
            if (RunLine())
            {
                // Some error occurred
                System.out.println("Stopped");
                break;
            }

            // prepare to process the next line
            cs.incrementPrgrmPosition();

            // stop if somebody pressed a key
            if (!cs.getPrgmRunning())
            {
                break;
            }
        }
        // v6.0.7 - 22 Jul 2015
        //ProcessPacket(c.ProcessKey(-1));
    }

    /*
     * This is a tweaked version from fmMain that removes the sleep delays
     */
    private boolean RunLine()
    {
        String line;
        int k1, k2, k3;
        final DisplayPacket p;

        // A quick sanity check
        // v6.0.2 - 26 Apr 12
        if (cs.getPrgmPosition() < 0 || cs.getPrgmPosition() >= cs.getPrgmMemory().size())
        {
            // if you "step off the edge", then just stop
            return true;
        }

        // get the current line
        line = cs.getPrgmMemory().get(cs.getPrgmPosition());
        //System.out.println("pos=" + (cs.getPrgmPosition()+1) + ", line=" + line);

        /*
         * This is kinda strange... there are 3 different formats for the program line.
         * They can have either 1 argument, 2 arguments, or 3 arguments.
         */
        if (line.startsWith("      "))  // one argument
        {
            try
            {
                k1 = Integer.parseInt(line.substring(6, 8).trim(), 16);
                p = c.ProcessKey(k1);
            }
            catch (Exception ex)
            {
                // I'm anticipating that folks might edit the XML by hand
                // to make minor tweaks to a program. So, we have to be
                // ready for a corrupted file
//                alertDialog(R.string.error_prgm_title, getString(R.string.error_prgm_msg) + cs.getPrgmPosition() + "\n" + ex.toString());
                return true;
            }
        }
        else if (line.startsWith("   "))  // two arguments
        {
            try
            {
                // v7.0.0 - 6 Jun 21: Oops, I missed this
                // special case for STO and RCL that optionally use the "." prefix
                if (line.charAt(6) == '.')
                {
                    k1 = Integer.parseInt(line.substring(3, 5).trim(), 16);
                    k2 = Integer.parseInt(line.substring(7, 8).trim(), 16);
                    c.ProcessKey(k1);
                    c.ProcessKey(Calculator.k.KeyDp.getVal());
                    p = c.ProcessKey(k2);
                }
                else
                {
                    k1 = Integer.parseInt(line.substring(3, 5).trim(), 16);
                    k2 = Integer.parseInt(line.substring(6, 8).trim(), 16);
                    c.ProcessKey(k1);
                    p = c.ProcessKey(k2);
                }
            }
            catch (Exception ex)
            {
//                alertDialog(R.string.error_prgm_title, getString(R.string.error_prgm_msg) + cs.getPrgmPosition() + "\n" + ex.toString());
                return true;
            }
        }
        else  // Three arguments
        {
            try
            {
                k1 = Integer.parseInt(line.substring(0, 2).trim(), 16);
                k2 = Integer.parseInt(line.substring(3, 5).trim(), 16);
                k3 = Integer.parseInt(line.substring(6, 8).trim(), 16);
                c.ProcessKey(k1);
                c.ProcessKey(k2);
                p = c.ProcessKey(k3);
            }
            catch (Exception ex)
            {
//                alertDialog(R.string.error_prgm_title, getString(R.string.error_prgm_msg) + cs.getPrgmPosition() + "\n" + ex.toString());
                return true;
            }
        }

        // We only update the display if there is a pause. No Annunciator
        // flags are updated while a program is running.
//        if (p.getDelay() > 0)
//        {
//            if (p.getAlternateText().isEmpty())
//            {
//                SetDisplayText(p.getDisplayText());
//                try
//                {
//                    Thread.sleep(p.getDelay());
//                }
//                catch (Exception ignore) {}
//            }
//            else
//            {
//                SetDisplayText(p.getAlternateText());
//                try
//                {
//                    Thread.sleep(p.getDelay());
//                }
//                catch (Exception ignore) {}
//            }
//        }
        return false;
    }
}
