package com.emmetgray.wrpn;

import org.junit.Test;

import static org.junit.Assert.*;

public class Row1
{
    private CalcState cs;
    private Calculator c;

    //  A,  B,  C,  D,  E,  F, 7,  8,  9, 10
    // 21, 22, 23, 24, 25, 26, 4,  5,  6, 20
    // 31, 32, 33, 35, 35, 36, 1,  2,  3, 30
    // 41, 42, 43, 44, 45,     0, 48, 49, 40

    private final int KeyA = 0xa;
    private final int KeyB = 0xb;
    private final int KeyC = 0xc;
    private final int KeyD = 0xd;
    private final int KeyE = 0xe;
    private final int KeyF = 0xf;
    private final int Key7 = 0x7;
    private final int Key8 = 0x8;
    private final int Key9 = 0x9;
    private final int KeyDiv = 0x10;
    private final int KeyGSB = 0x21;
    private final int KeyGTO = 0x22;
    private final int KeyHEX = 0x23;
    private final int KeyDEC = 0x24;
    private final int KeyOCT = 0x25;
    private final int KeyBIN = 0x26;
    private final int Key4 = 0x4;
    private final int Key5 = 0x5;
    private final int Key6 = 0x6;
    private final int KeyMul = 0x20;
    private final int KeyRS = 0x31;
    private final int KeySST = 0x32;
    private final int KeyRol = 0x33;
    private final int KeyXY = 0x34;
    private final int KeyBSP = 0x35;
    private final int KeyEnt = 0x36;
    private final int Key1 = 0x1;
    private final int Key2 = 0x2;
    private final int Key3 = 0x3;
    private final int KeyMin = 0x30;
    private final int KeyON = 0x41;
    private final int KeyFKey = 0x42;
    private final int KeyGKey = 0x43;
    private final int KeySTO = 0x44;
    private final int KeyRCL = 0x45;
    private final int Key0 = 0x0;
    private final int KeyDp = 0x48;
    private final int KeyCHS = 0x49;
    private final int KeyAdd = 0x40;

    public Row1()
    {
        cs = new CalcState("/storage/emulated/0/Android/data/com.emmetgray.wrpn/files/wrpn.config");
        c = new Calculator(cs);
    }

    /*
     * Test all of the yellow-shifted function on the first row.  Uses both positive and
     * negative numbers and all 3 arithmetic modes
     */
    @Test
    public void row1Yellow()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);

        /*
         * Round 1... using positive numbers and 1's complement
         */
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyA);    // SL
        assertEquals("SL", "6 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyB);    // SR
        assertEquals("SR", "1 d", dp.getDisplayText());
        assertEquals("SR Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyC);    // RL
        assertEquals("RL", "6 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyD);    // RR
        assertEquals("RR", "-32766 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyE);    // RLn
        assertEquals("RLn", "12 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyF);    // RRn
        assertEquals("RRn", "-16383 d", dp.getDisplayText());
        assertEquals("RRn Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(7);    // MASKL
        assertEquals("MASKL", "-8191 d", dp.getDisplayText());
        assertEquals("MASKL Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(8);    // MASKR
        assertEquals("MASKR", "7 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(9);    // RMD
        assertEquals("RMD", "1 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDiv);    // XOR
        assertEquals("XOR", "1 d", dp.getDisplayText());

        /*
         * Round 2... using negative numbers (and 1's complement)
         */
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyA);    // SL
        assertEquals("SL", "-7 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyB);    // SR
        assertEquals("SR", "32766 d", dp.getDisplayText());
        assertEquals("SR Carry", false, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyC);    // RL
        assertEquals("RL", "-6 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyD);    // RR
        assertEquals("RR", "32766 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyE);    // RLn
        assertEquals("RLn", "-12 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyF);    // RRn
        assertEquals("RRn", "16383 d", dp.getDisplayText());
        assertEquals("RRn Carry", false, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(7);    // MASKL
        assertEquals("MASKL", "-8191 d", dp.getDisplayText());
        assertEquals("MASKL Carry", false, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(8);    // MASKR
        assertEquals("MASKR", "7 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(9);    // RMD
        assertEquals("RMD", "-1 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyCHS);   // switched which one is neg
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(9);    // RMD
        assertEquals("RMD", "1 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDiv);    // XOR
        assertEquals("XOR", "-1 d", dp.getDisplayText());

        /*
         * Round 3 ... using positive numbers and 2's complement
         */
        // 2's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(2);

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyA);    // SL
        assertEquals("SL", "6 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyB);    // SR
        assertEquals("SR", "1 d", dp.getDisplayText());
        assertEquals("SR Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyC);    // RL
        assertEquals("RL", "6 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyD);    // RR
        assertEquals("RR", "-32767 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyE);    // RLn
        assertEquals("RLn", "12 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyF);    // RRn
        assertEquals("RRn", "-16384 d", dp.getDisplayText());
        assertEquals("RRn Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(7);    // MASKL
        assertEquals("MASKL", "-8192 d", dp.getDisplayText());
        assertEquals("MASKL Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(8);    // MASKR
        assertEquals("MASKR", "7 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(9);    // RMD
        assertEquals("RMD", "1 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDiv);    // XOR
        assertEquals("XOR", "1 d", dp.getDisplayText());

        /*
         * Round 4... using negative numbers (and 2's complement)
         */
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyA);    // SL
        assertEquals("SL", "-6 d", dp.getDisplayText());
        assertEquals("SL Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyB);    // SR
        assertEquals("SR", "32766 d", dp.getDisplayText());
        assertEquals("SR Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyC);    // RL
        assertEquals("RL", "-5 d", dp.getDisplayText());
        assertEquals("RL Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyD);    // RR
        assertEquals("RR", "-2 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyE);    // RLn
        assertEquals("RLn", "-9 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyF);    // RRn
        assertEquals("RRn", "32767 d", dp.getDisplayText());
        assertEquals("RRn Carry", false, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(7);    // MASKL
        assertEquals("MASKL", "-8192 d", dp.getDisplayText());
        assertEquals("MASKL Carry", false, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(8);    // MASKR
        assertEquals("MASKR", "7 d", dp.getDisplayText() );

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(9);    // RMD
        assertEquals("RMD", "-1 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyCHS);   // switched which one is neg
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(9);    // RMD
        assertEquals("RMD", "1 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDiv);    // XOR
        assertEquals("XOR", "-1 d", dp.getDisplayText() );

        /*
         * Round 5... unsigned (no need to test for negative numbers)
         */
        // unsigned mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(3);

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyA);    // SL
        assertEquals("SL", "6 d", dp.getDisplayText() );

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyB);    // SR
        assertEquals("SR", dp.getDisplayText(), "1 d");
        assertEquals("SR Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyC);    // RL
        assertEquals("RL", "6 d", dp.getDisplayText() );

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyD);    // RR
        assertEquals("RR", "32769 d", dp.getDisplayText() );

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyE);    // RLn
        assertEquals("RLn", "12 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyF);    // RRn
        assertEquals("RRn", "49152 d", dp.getDisplayText());
        assertEquals("RRn Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(7);    // MASKL
        assertEquals("MASKL", "57344 d", dp.getDisplayText());
        assertEquals("MASKL Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(8);    // MASKR
        assertEquals("MASKR", "7 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(9);    // RMD
        assertEquals("RMD", "1 d", dp.getDisplayText() );

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDiv);    // XOR
        assertEquals("XOR", "1 d", dp.getDisplayText() );
    }

    @Test
    public void row1Blue()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);

        /*
         * Round 1... using positive numbers and 1's complement
         */
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyA);    // LF
        assertEquals("LF", "14 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("LF", "-16383 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyB);    // ASR
        assertEquals("ASR", "1 d", dp.getDisplayText());
        assertEquals("ASR Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC1", "6 d", dp.getDisplayText());
        assertEquals("RLC1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC2", "7 d", dp.getDisplayText());
        assertEquals("RLC2 Carry", false, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC1", "1 d", dp.getDisplayText());
        assertEquals("RRC1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC2", "-32766 d", dp.getDisplayText());
        assertEquals("RRC2 Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn1", "12 d", dp.getDisplayText());
        assertEquals("RLCn1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn2", "14 d", dp.getDisplayText());
        assertEquals("RLCn2 Carry", false, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn1", "-32767 d", dp.getDisplayText());
        assertEquals("RRCn1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn2", "-16383 d", dp.getDisplayText());
        assertEquals("RRCn2 Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(7);    // #B
        assertEquals("#B", "2 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(8);    // ABS
        assertEquals("ABS", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDiv);    // DBLd
        assertEquals("DBLd", "8192 d", dp.getDisplayText());
        assertEquals("DBLd Carry", true, dp.getCarry_Annunciator());

        /*
         * Round 2... using negative numbers and 1's complement
         */

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyA);    // LF
        assertEquals("LF", "0 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("LF", "-3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyB);    // ASR
        assertEquals("ASR", "-1 d", dp.getDisplayText());
        assertEquals("ASR Carry", false, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC1", "-7 d", dp.getDisplayText());
        assertEquals("RLC1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC2", "-6 d", dp.getDisplayText());
        assertEquals("RLC2 Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC1", "32766 d", dp.getDisplayText());
        assertEquals("RRC1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC2", "-1 d", dp.getDisplayText());
        assertEquals("RRC2 Carry", false, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn1", "-14 d", dp.getDisplayText());
        assertEquals("RLCn1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn2", "-12 d", dp.getDisplayText());
        assertEquals("RLCn2 Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn1", "16383 d", dp.getDisplayText());
        assertEquals("RRCn1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn2", "32767 d", dp.getDisplayText());
        assertEquals("RRCn2 Carry", false, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(7);    // #B
        assertEquals("#B", "14 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(8);    // ABS
        assertEquals("ABS", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR1", "12 d", dp.getDisplayText());

        c.ProcessKey(3);
         c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyCHS);   // reversed
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR2", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyCHS);   // reversed
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR3", "-12 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDiv);    // DBLd
        assertEquals("DBLd1", "12287 d", dp.getDisplayText());
        assertEquals("DBLd1 Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDiv);    // DBLd
        assertEquals("DBLd2", "-12287 d", dp.getDisplayText());
        assertEquals("DBLd2 Carry", true, dp.getCarry_Annunciator());

        /*
         * Round 3... using positive numbers and 2's complement
         */
        // 3's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(2);

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyA);    // LF
        assertEquals("LF", "14 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("LF", "-16384 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyB);    // ASR
        assertEquals("ASR", "1 d", dp.getDisplayText());
        assertEquals("ASR Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC1", "6 d", dp.getDisplayText());
        assertEquals("RLC1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC2", "7 d", dp.getDisplayText());
        assertEquals("RLC2 Carry", false, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC1", "1 d", dp.getDisplayText());
        assertEquals("RRC1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC2", "-32767 d", dp.getDisplayText());
        assertEquals("RRC2 Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn1", "12 d", dp.getDisplayText());
        assertEquals("RLCn1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn2", "14 d", dp.getDisplayText());
        assertEquals("RLCn2 Carry", false, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn1", "-32768 d", dp.getDisplayText());
        assertEquals("RRCn1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn2", "-16384 d", dp.getDisplayText());
        assertEquals("RRCn2 Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(7);    // #B
        assertEquals("#B", "2 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(8);    // ABS
        assertEquals("ABS", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDiv);    // DBLd
        assertEquals("DBLd", "8192 d", dp.getDisplayText());
        assertEquals("DBLd Carry", true, dp.getCarry_Annunciator());

        /*
         * Round 4... using negative numbers and 2's complement
         */

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyA);    // LF
        assertEquals("LF", "0 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("LF", "-3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyB);    // ASR
        assertEquals("ASR", "-2 d", dp.getDisplayText());
        assertEquals("ASR Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC1", "-6 d", dp.getDisplayText());
        assertEquals("RLC1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC2", "-5 d", dp.getDisplayText());
        assertEquals("RLC2 Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC1", "32766 d", dp.getDisplayText());
        assertEquals("RRC1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC2", "-2 d", dp.getDisplayText());
        assertEquals("RRC2 Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn1", "-11 d", dp.getDisplayText());
        assertEquals("RLCn1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn2", "-9 d", dp.getDisplayText());
        assertEquals("RLCn2 Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn1", "-16385 d", dp.getDisplayText());
        assertEquals("RRCn1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn2", "-1 d", dp.getDisplayText());
        assertEquals("RRCn2 Carry", false, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(7);    // #B
        assertEquals("#B", "15 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(8);    // ABS
        assertEquals("ABS", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR1", "13 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyCHS);   // reversed
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR2", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyCHS);   // reversed
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR3", "-13 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDiv);    // DBLd
        assertEquals("DBLd1", "12287 d", dp.getDisplayText());
        assertEquals("DBLd1 Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDiv);    // DBLd
        assertEquals("DBLd2", "-8191 d", dp.getDisplayText());
        assertEquals("DBLd2 Carry", true, dp.getCarry_Annunciator());

        /*
         * Round 5... unsigned (no need to test for negative numbers)
         */
        // unsigned mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(3);

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyA);    // LF
        assertEquals("LF", "14 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("LF", "49152 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyB);    // ASR
        assertEquals("ASR", "1 d", dp.getDisplayText());
        assertEquals("ASR Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC1", "6 d", dp.getDisplayText());
        assertEquals("RLC1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyC);    // RLC
        assertEquals("RLC2", "7 d", dp.getDisplayText());
        assertEquals("RLC2 Carry", false, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC1", "1 d", dp.getDisplayText());
        assertEquals("RRC1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyD);    // RRC
        assertEquals("RRC2", "32769 d", dp.getDisplayText());
        assertEquals("RRC2 Carry", true, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn1", "12 d", dp.getDisplayText());
        assertEquals("RLCn1 Carry", false, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyE);    // RLCn
        assertEquals("RLCn2", "14 d", dp.getDisplayText());
        assertEquals("RLCn2 Carry", false, dp.getCarry_Annunciator());

        clearCarry();
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn1", "32768 d", dp.getDisplayText());
        assertEquals("RRCn1 Carry", true, dp.getCarry_Annunciator());

        setCarry(); // again, but with carry bit set
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyF);    // RRCn
        assertEquals("RRCn2", "49152 d", dp.getDisplayText());
        assertEquals("RRCn2 Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(7);    // #B
        assertEquals("#B", "2 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(8);    // ABS
        assertEquals("ABS", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(9);    // DBLR
        assertEquals("DBLR", "3 d", dp.getDisplayText());

        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDiv);    // DBLd
        assertEquals("DBLd", "8192 d", dp.getDisplayText());
        assertEquals("DBLd Carry", true, dp.getCarry_Annunciator());
    }

    @Test
    public void row1White()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);

        /*
         * Round 1... using positive numbers and 1's complement
         */

        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        dp = c.ProcessKey(KeyDiv);
        assertEquals("Div", "5 d", dp.getDisplayText());
        assertEquals("Div Carry", true, dp.getCarry_Annunciator());

        /*
         * Round 2... using negative numbers and 1's complement
         */

        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        dp = c.ProcessKey(KeyDiv);
        assertEquals("Div", "-5 d", dp.getDisplayText());
        assertEquals("Div Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        dp = c.ProcessKey(KeyDiv);
        assertEquals("Div", "-5 d", dp.getDisplayText());
        assertEquals("Div Carry", true, dp.getCarry_Annunciator());

        /*
         * Round 3... using positive numbers and 2's complement
         */
        // 2's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(2);

        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        dp = c.ProcessKey(KeyDiv);
        assertEquals("Div", "5 d", dp.getDisplayText());
        assertEquals("Div Carry", true, dp.getCarry_Annunciator());

        /*
         * Round 4... using negative numbers and 2's complement
         */

        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        dp = c.ProcessKey(KeyDiv);
        assertEquals("Div", "-5 d", dp.getDisplayText());
        assertEquals("Div Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);
        dp = c.ProcessKey(KeyDiv);
        assertEquals("Div", "-5 d", dp.getDisplayText());
        assertEquals("Div Carry", true, dp.getCarry_Annunciator());

        /*
         * Round 5... positive numbers and unsigned
         */

        // unsigned mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(3);

        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        dp = c.ProcessKey(KeyDiv);
        assertEquals("Div", "5 d", dp.getDisplayText());
        assertEquals("Div Carry", true, dp.getCarry_Annunciator());
    }

    private void clearCarry()
    {
        c.ProcessKey(KeyGKey);
        c.ProcessKey(5);
        c.ProcessKey(4);
    }
    private void setCarry()
    {
        c.ProcessKey(KeyGKey);
        c.ProcessKey(4);
        c.ProcessKey(4);
    }
}