package com.emmetgray.wrpn;

import org.junit.Test;

import static org.junit.Assert.*;

public class Row2
{
    private CalcState cs;
    private Calculator c;
    private Prgm p;

    //  A,  B,  C,  D,  E,  F, 7,  8,  9, 10
    // 21, 22, 23, 24, 25, 26, 4,  5,  6, 20
    // 31, 32, 33, 35, 35, 36, 1,  2,  3, 30
    // 41, 42, 43, 44, 45,     0, 48, 49, 40

    private final int KeyA = 0xa;
    private final int KeyB = 0xb;
    private final int KeyC = 0xc;
    private final int KeyD = 0xd;
    private final int KeyE = 0xe;
    private final int KeyF = 0xf;
    private final int Key7 = 0x7;
    private final int Key8 = 0x8;
    private final int Key9 = 0x9;
    private final int KeyDiv = 0x10;
    private final int KeyGSB = 0x21;
    private final int KeyGTO = 0x22;
    private final int KeyHEX = 0x23;
    private final int KeyDEC = 0x24;
    private final int KeyOCT = 0x25;
    private final int KeyBIN = 0x26;
    private final int Key4 = 0x4;
    private final int Key5 = 0x5;
    private final int Key6 = 0x6;
    private final int KeyMul = 0x20;
    private final int KeyRS = 0x31;
    private final int KeySST = 0x32;
    private final int KeyRol = 0x33;
    private final int KeyXY = 0x34;
    private final int KeyBSP = 0x35;
    private final int KeyEnt = 0x36;
    private final int Key1 = 0x1;
    private final int Key2 = 0x2;
    private final int Key3 = 0x3;
    private final int KeyMin = 0x30;
    private final int KeyON = 0x41;
    private final int KeyFKey = 0x42;
    private final int KeyGKey = 0x43;
    private final int KeySTO = 0x44;
    private final int KeyRCL = 0x45;
    private final int Key0 = 0x0;
    private final int KeyDp = 0x48;
    private final int KeyCHS = 0x49;
    private final int KeyAdd = 0x40;

    public Row2()
    {
        cs = new CalcState("/storage/emulated/0/Android/data/com.emmetgray.wrpn/files/wrpn.config");
        c = new Calculator(cs);
        p = new Prgm(cs, c);
    }

    /*
     * Test all of the yellow-shifted function on the second row.
     */
    @Test
    public void row2Yellow()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);

        /*
         * Round 1... using positive numbers and 1's complement
         */
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(5);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySST);       // store 5 in Reg index
        c.ProcessKey(3);
        c.ProcessKey(KeySTO);
        c.ProcessKey(5);    // put a 3 in Reg 5
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyGSB);       // x:(i)
        c.ProcessKey(KeyRCL);
        dp = c.ProcessKey(5);
        assertEquals("x:(i)", "3 d", dp.getDisplayText());
        c.ProcessKey(KeyRCL);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeySST);
        assertEquals("x:(i)", "5 d", dp.getDisplayText());

        c.ProcessKey(5);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySST);       // store 5 in Reg index
        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyGTO);       // x:I
        assertEquals("x:I", "5 d", dp.getDisplayText());
        c.ProcessKey(KeyRCL);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeySST);
        assertEquals("x:I", "3 d", dp.getDisplayText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(4);
        c.ProcessKey(3);    // enable leading zeros
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyHEX);
        assertEquals("ShowHex", "0011 h", dp.getAlternateText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(5);
        c.ProcessKey(3);    // disable leading zeros
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyHEX);
        assertEquals("ShowHex", "11 h", dp.getAlternateText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(4);
        c.ProcessKey(3);    // enable leading zeros
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyDEC);
        assertEquals("ShowDec", "17 d", dp.getAlternateText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(5);
        c.ProcessKey(3);    // disable leading zeros
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyDEC);
        assertEquals("ShowDec", "17 d", dp.getAlternateText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(4);
        c.ProcessKey(3);    // enable leading zeros
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyOCT);
        assertEquals("ShowOct", "000021 o", dp.getAlternateText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(5);
        c.ProcessKey(3);    // disable leading zeros
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyOCT);
        assertEquals("ShowOct", "21 o", dp.getAlternateText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(4);
        c.ProcessKey(3);    // enable leading zeros
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyBIN);
        assertEquals("ShowBin", "00000000 00010001 b", dp.getAlternateText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(5);
        c.ProcessKey(3);    // disable leading zeros
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyBIN);
        assertEquals("ShowBin", "10001 b", dp.getAlternateText());

        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(4);    // bit 4
        c.ProcessKey(KeyFKey);
        c.ProcessKey(4);
        assertEquals("SB", "17 d", dp.getDisplayText());

        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);    // bit 5
        c.ProcessKey(KeyFKey);
        c.ProcessKey(4);
        assertEquals("SB", "49 d", dp.getDisplayText());

        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(4);    // bit 4
        c.ProcessKey(KeyFKey);
        c.ProcessKey(5);
        assertEquals("CB", "1 d", dp.getDisplayText());

        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);    // bit 5
        c.ProcessKey(KeyFKey);
        c.ProcessKey(5);
        assertEquals("CB", "17 d", dp.getDisplayText());

        // test programming functions via the Program Position (increment on false)
        cs.setPrgmPosition(0);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(4);    // bit 4
        c.ProcessKey(KeyFKey);
        c.ProcessKey(6);
        assertEquals("B?", 0, cs.getPrgmPosition());

        cs.setPrgmPosition(0);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(7);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);    // bit 5
        c.ProcessKey(KeyFKey);
        c.ProcessKey(6);
        assertEquals("B?", 1, cs.getPrgmPosition());

        c.ProcessKey(4);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyMul);
        assertEquals("AND", "0 d", dp.getDisplayText());

        c.ProcessKey(4);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyMul);
        assertEquals("AND", "4 d", dp.getDisplayText());

        /*
         * Negative Numbers...
         */
        c.ProcessKey(4);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyMul);
        assertEquals("AND", "3 d", dp.getDisplayText());

        // negative numbers
        c.ProcessKey(4);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);    // swapped
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyMul);
        assertEquals("AND", "4 d", dp.getDisplayText());

        // 2's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(2);

        c.ProcessKey(4);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyMul);
        assertEquals("AND", "0 d", dp.getDisplayText());

        // negative numbers
        c.ProcessKey(4);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyCHS);    // swapped
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyMul);
        assertEquals("AND", "4 d", dp.getDisplayText());
    }

    @Test
    public void row2Blue()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);

        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        /*
         * Run the "simple program" to test GSB, GTO, RTN, LBL, PSE, DSZ, etc
         */
        p.simpleProgram();
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyRol);           // reset to 000
        p.run();
        dp = c.ProcessKey(-1);
        assertEquals("PRGM", "15 d", dp.getDisplayText());

        cs.setPrgmPosition(5);
        c.ProcessKey(2);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyGTO);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDEC);
        assertEquals("ISZ1", 5, cs.getPrgmPosition());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDEC);
        assertEquals("ISZ2", 6, cs.getPrgmPosition());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDEC);      // did not advance = true
        assertEquals("ISZ3", 6, cs.getPrgmPosition());

        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyOCT);
        assertEquals("SQRT", "2 d", dp.getDisplayText());
        assertEquals("SQRT Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(5);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyOCT);
        assertEquals("SQRT", "Error 0 - Improper Math Operation", dp.getAlternateText());
        assertEquals("SQRT Carry", true, dp.getCarry_Annunciator());

        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyBIN);  // just ignored...
        assertEquals("1/x", "5 d", dp.getDisplayText());

        c.ProcessKey(KeyGKey);
        c.ProcessKey(4);
        dp = c.ProcessKey(0);
        assertEquals("SF", true, cs.getFlag(CalcState.CalcFlag.User0));

        c.ProcessKey(KeyGKey);
        c.ProcessKey(5);
        dp = c.ProcessKey(0);
        assertEquals("CF", false, cs.getFlag(CalcState.CalcFlag.User0));

        cs.setPrgmPosition(5);
        c.ProcessKey(KeyGKey);
        c.ProcessKey(6);
        dp = c.ProcessKey(0);
        assertEquals("F?", 6, cs.getPrgmPosition());

        cs.setPrgmPosition(5);
        c.ProcessKey(KeyGKey);
        c.ProcessKey(4);
        c.ProcessKey(0);    // set it again..
        c.ProcessKey(KeyGKey);
        c.ProcessKey(6);
        dp = c.ProcessKey(0);  // did not advance = true
        assertEquals("F?", 5, cs.getPrgmPosition());

        // 1's comp
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(3);
        c.ProcessKey(2);
        c.ProcessKey(0);
        c.ProcessKey(0);
        c.ProcessKey(0);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(5);    // 32000 x 15 = 480000
        dp = c.ProcessKey(KeyMul);
        assertEquals("Mul", "21248 d", dp.getDisplayText());
        assertEquals("Mul Overflow", true, dp.getOverflow_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(2);
        c.ProcessKey(0);
        c.ProcessKey(0);
        c.ProcessKey(0);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(5);    // 32000 x 15 = 480000
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyMul);
        assertEquals("DBL*", "7 d", dp.getDisplayText());
        c.ProcessKey(KeyRol);
        assertEquals("DBL*", "21248 d", dp.getDisplayText());
        assertEquals("DBL* Overflow", false, dp.getOverflow_Annunciator());

        c.ProcessKey(3);
        c.ProcessKey(2);
        c.ProcessKey(0);
        c.ProcessKey(0);
        c.ProcessKey(0);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        c.ProcessKey(5);    // 32000 x 15 = 480000
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyMul);
        assertEquals("DBL*", "-7 d", dp.getDisplayText());
        c.ProcessKey(KeyRol);
        assertEquals("DBL*", "-21248 d", dp.getDisplayText());
        assertEquals("DBL* Overflow", false, dp.getOverflow_Annunciator());
    }
}
