package com.emmetgray.wrpn;

import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class Row3
{
    private CalcState cs;
    private Calculator c;
    private Prgm p;

    //  A,  B,  C,  D,  E,  F, 7,  8,  9, 10
    // 21, 22, 23, 24, 25, 26, 4,  5,  6, 20
    // 31, 32, 33, 35, 35, 36, 1,  2,  3, 30
    // 41, 42, 43, 44, 45,     0, 48, 49, 40

    private final int KeyA = 0xa;
    private final int KeyB = 0xb;
    private final int KeyC = 0xc;
    private final int KeyD = 0xd;
    private final int KeyE = 0xe;
    private final int KeyF = 0xf;
    private final int Key7 = 0x7;
    private final int Key8 = 0x8;
    private final int Key9 = 0x9;
    private final int KeyDiv = 0x10;
    private final int KeyGSB = 0x21;
    private final int KeyGTO = 0x22;
    private final int KeyHEX = 0x23;
    private final int KeyDEC = 0x24;
    private final int KeyOCT = 0x25;
    private final int KeyBIN = 0x26;
    private final int Key4 = 0x4;
    private final int Key5 = 0x5;
    private final int Key6 = 0x6;
    private final int KeyMul = 0x20;
    private final int KeyRS = 0x31;
    private final int KeySST = 0x32;
    private final int KeyRol = 0x33;
    private final int KeyXY = 0x34;
    private final int KeyBSP = 0x35;
    private final int KeyEnt = 0x36;
    private final int Key1 = 0x1;
    private final int Key2 = 0x2;
    private final int Key3 = 0x3;
    private final int KeyMin = 0x30;
    private final int KeyON = 0x41;
    private final int KeyFKey = 0x42;
    private final int KeyGKey = 0x43;
    private final int KeySTO = 0x44;
    private final int KeyRCL = 0x45;
    private final int Key0 = 0x0;
    private final int KeyDp = 0x48;
    private final int KeyCHS = 0x49;
    private final int KeyAdd = 0x40;

    public Row3()
    {
        cs = new CalcState("/storage/emulated/0/Android/data/com.emmetgray.wrpn/files/wrpn.config");
        c = new Calculator(cs);
        p = new Prgm(cs, c);
    }

    @Test
    public void row3Yellow()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        // Note: (i) and I have already been covered by the programming test

        cs.setPrgmPosition(5);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyRol);
        assertEquals("cl Prgm", 0, cs.getPrgmPosition());

        p.simpleProgram();
        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyRS);        // enter program mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyRol);
        c.ProcessKey(KeyGKey);
        c.ProcessKey(KeyRS);        // exit
        assertEquals("cl Prgm", 0, cs.getPrgmMemory().size());

        c.ProcessKey(5);
        c.ProcessKey(KeySTO);
        c.ProcessKey(5);    // put a 5 in Reg 5
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyXY);

        for (int i=0; i<32; i++)
        {
            c.ProcessKey(KeyRCL);
            dp = c.ProcessKey(i);
            assertEquals("cl Reg", "0 d", dp.getDisplayText());
        }

        c.ProcessKey(5);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);  // clear the STO "prefix" key
        c.ProcessKey(KeyBSP);
        dp = c.ProcessKey(3);
        assertEquals("cl Prefix", "53 d", dp.getDisplayText());

        /*
         * Change word size
         */
        // decimal mode
        c.ProcessKey(KeyDEC);
        // 64 bit word size
        c.ProcessKey(6);
        c.ProcessKey(4);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-64-1000", dp.getAlternateText());

        c.ProcessKey(2);    // 2 billion
        for (int i=0; i<9; i++)
        {
            c.ProcessKey(0);
        }
        c.ProcessKey(KeyEnt);
        c.ProcessKey(4);
        c.ProcessKey(0);
        c.ProcessKey(0);    // 4 hundred
        dp = c.ProcessKey(KeyMul);
        assertEquals("Mul", "800000000000 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyBIN);
        assertEquals("win0", "01000011 10110111 01000000 00000000 .b", dp.getDisplayText());
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyEnt);
        dp = c.ProcessKey(1);
        assertEquals("win1", "00000000 00000000 00000000 10111010 b.", dp.getDisplayText());

        // I think 1's, 2's, and unsigned have already been tested enough

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-16-1000", dp.getAlternateText());

        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyMin);
        assertEquals("Not", "-3 d", dp.getDisplayText());
    }

    @Test
    public void row3Blue()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        // no need to test P/R here.. is already covered by SimpleProgram

        p.simpleProgram();
        cs.setPrgmPosition(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeySST);
        assertEquals("BST", 4, cs.getPrgmPosition());

        c.ProcessKey(1);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(2);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(4);    // fill the stack
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyRol);
        assertEquals("RolUp", "1 d", dp.getDisplayText());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyRol);
        assertEquals("RolUp", "2 d", dp.getDisplayText());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyRol);
        assertEquals("RolUp", "3 d", dp.getDisplayText());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyRol);
        assertEquals("RolUp", "4 d", dp.getDisplayText());

        // difficult to test PSE in a Unit Test...

        c.ProcessKey(1);
        c.ProcessKey(2);
        c.ProcessKey(3);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyBSP);
        assertEquals("CLx", "0 d", dp.getDisplayText());
        // this part needs the test above...
        dp = c.ProcessKey(KeyRol);
        assertEquals("Rol", "4 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("Rol", "3 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("Rol", "2 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("Rol", "0 d", dp.getDisplayText());

        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key1);
        assertEquals("x<=y", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key1);
        assertEquals("x<=y", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(6);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key1);
        assertEquals("x<=y", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key1);
        assertEquals("x<=y", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key1);
        assertEquals("x<=y", 6, cs.getPrgmPosition());

        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key2);
        assertEquals("x<0", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(0);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key2);
        assertEquals("x<0", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(1);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key2);
        assertEquals("x<0", 5, cs.getPrgmPosition());

        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key3);
        assertEquals("x>y", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key3);
        assertEquals("x>y", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(6);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key3);
        assertEquals("x>y", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key3);
        assertEquals("x>y", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key3);
        assertEquals("x>y", 5, cs.getPrgmPosition());

        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyMin);
        assertEquals("x>0", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(0);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyMin);
        assertEquals("x>0", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(1);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyMin);
        assertEquals("x>0", 6, cs.getPrgmPosition());
    }

    @Test
    public void row3White()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        p.simpleProgram();
        cs.setPrgmPosition(5);
        dp = c.ProcessKey(KeySST);
        assertEquals("SST", 6, cs.getPrgmPosition());

        // Rol has already been covered

        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        dp = c.ProcessKey(KeyXY);
        assertEquals("XY", "5 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("XY", "6 d", dp.getDisplayText());

        c.ProcessKey(1);
        c.ProcessKey(2);
        c.ProcessKey(3);
        dp = c.ProcessKey(KeyBSP);
        assertEquals("BSP", "12 d", dp.getDisplayText());

        c.ProcessKey(KeyEnt);
        c.ProcessKey(1);
        dp = c.ProcessKey(KeyBSP);       // no harm doing it twice
        assertEquals("BSP", "0 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyBSP);
        assertEquals("BSP", "0 d", dp.getDisplayText());

        // I think this is already covered...
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        dp = c.ProcessKey(KeyMin);
        assertEquals("Min", "-1 d", dp.getDisplayText());
    }
}
