package com.emmetgray.wrpn;

import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class Row4
{
    private CalcState cs;
    private Calculator c;
    private Prgm p;

    //  A,  B,  C,  D,  E,  F, 7,  8,  9, 10
    // 21, 22, 23, 24, 25, 26, 4,  5,  6, 20
    // 31, 32, 33, 35, 35, 36, 1,  2,  3, 30
    // 41, 42, 43, 44, 45,     0, 48, 49, 40
    
    private final int KeyA = 0xa;
    private final int KeyB = 0xb;
    private final int KeyC = 0xc;
    private final int KeyD = 0xd;
    private final int KeyE = 0xe;
    private final int KeyF = 0xf;
    private final int Key7 = 0x7;
    private final int Key8 = 0x8;
    private final int Key9 = 0x9;
    private final int KeyDiv = 0x10;
    private final int KeyGSB = 0x21;
    private final int KeyGTO = 0x22;
    private final int KeyHEX = 0x23;
    private final int KeyDEC = 0x24;
    private final int KeyOCT = 0x25;
    private final int KeyBIN = 0x26;
    private final int Key4 = 0x4;
    private final int Key5 = 0x5;
    private final int Key6 = 0x6;
    private final int KeyMul = 0x20;
    private final int KeyRS = 0x31;
    private final int KeySST = 0x32;
    private final int KeyRol = 0x33;
    private final int KeyXY = 0x34;
    private final int KeyBSP = 0x35;
    private final int KeyEnt = 0x36;
    private final int Key1 = 0x1;
    private final int Key2 = 0x2;
    private final int Key3 = 0x3;
    private final int KeyMin = 0x30;
    private final int KeyON = 0x41;
    private final int KeyFKey = 0x42;
    private final int KeyGKey = 0x43;
    private final int KeySTO = 0x44;
    private final int KeyRCL = 0x45;
    private final int Key0 = 0x0;
    private final int KeyDp = 0x48;
    private final int KeyCHS = 0x49;
    private final int KeyAdd = 0x40;

    public Row4()
    {
        cs = new CalcState("/storage/emulated/0/Android/data/com.emmetgray.wrpn/files/wrpn.config");
        c = new Calculator(cs);
        p = new Prgm(cs, c);
    }

    @Test
    public void row4Yellow()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-16-1000", dp.getAlternateText());

        c.ProcessKey(1);
        c.ProcessKey(5);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-15-1000", dp.getAlternateText());

        c.ProcessKey(6);
        c.ProcessKey(2);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-62-1000", dp.getAlternateText());

        c.ProcessKey(6);
        c.ProcessKey(4);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-64-1000", dp.getAlternateText());

        c.ProcessKey(0);    // special case for "escape clause"
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-64-1000", dp.getAlternateText());

        /*
         * My technique for doing scientific notation is a lot different
         */
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeyRCL);
        c.ProcessKey(3);    // now in float mode (with 3 decimal places)

        c.ProcessKey(2);
        dp = c.ProcessKey(KeyEnt);
        assertEquals("Float", "2.000", dp.getDisplayText());

        c.ProcessKey(4);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyCHS);
        assertEquals("EEX", "4e", dp.getDisplayText());
        dp = c.ProcessKey(3);
        assertEquals("EEX", "4e3", dp.getDisplayText());
        /*
         * This is very strange... this works "in real life", but fails under
         * the unit test.  I don't get it.
         */
//        dp = c.ProcessKey(KeyEnt);
//        assertEquals("EEX", "4000.000", dp.getDisplayText());

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-16-1000", dp.getAlternateText());

        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyAdd);
        assertEquals("OR", "7 d", dp.getDisplayText());
    }

    @Test
    public void row4Blue()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-16-1000", dp.getAlternateText());

        // 16 bit word size
        c.ProcessKey(6);
        c.ProcessKey(4);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-64-1000", dp.getAlternateText());

        /*
         * The scroll function operates radically different... it just moves the
         * entire screen over (to include the spaces between bytes)
         */
        c.ProcessKey(2);    // 2 billion
        for (int i=0; i<9; i++)
        {
            c.ProcessKey(0);
        }
        c.ProcessKey(KeyEnt);
        c.ProcessKey(4);
        c.ProcessKey(0);
        c.ProcessKey(0);    // 4 hundred
        dp = c.ProcessKey(KeyMul);
        assertEquals("Mul", "800000000000 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyBIN);
        assertEquals("Scroll <", "01000011 10110111 01000000 00000000 .b", dp.getDisplayText());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeySTO);
        assertEquals("Scroll <", "01000011 10110111 01000000 00000000 .b", dp.getDisplayText());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyRCL);
        assertEquals("Scroll >", " 01000011 10110111 01000000 0000000 .b.", dp.getDisplayText());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyRCL);
        assertEquals("Scroll >", "0 01000011 10110111 01000000 000000 .b.", dp.getDisplayText());
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeySTO);
        assertEquals("Scroll <", " 01000011 10110111 01000000 0000000 .b.", dp.getDisplayText());

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);

        c.ProcessKey(KeyFKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("status", "1-16-1000", dp.getAlternateText());

        c.ProcessKey(0);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(0);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(0);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(0);
        c.ProcessKey(KeyEnt);       // clear the stack

        c.ProcessKey(4);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(3);
        c.ProcessKey(KeyMul);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyEnt);
        assertEquals("LSTx", "3 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("LSTx", "12 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("LSTx", "0 d", dp.getDisplayText());
        dp = c.ProcessKey(KeyRol);
        assertEquals("LSTx", "0 d", dp.getDisplayText());

        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key0);
        assertEquals("x!=y", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key0);
        assertEquals("x!=y", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(0);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(Key0);
        assertEquals("x!=y", 5, cs.getPrgmPosition());

        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("x!=0", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("x!=0", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(0);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyDp);
        assertEquals("x!=0", 6, cs.getPrgmPosition());

        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyCHS);
        assertEquals("x=y", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyCHS);
        assertEquals("x=y", 5, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(0);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyCHS);
        assertEquals("x=y", 6, cs.getPrgmPosition());

        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyAdd);
        assertEquals("x=0", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(5);
        c.ProcessKey(KeyCHS);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyAdd);
        assertEquals("x=0", 6, cs.getPrgmPosition());
        cs.setPrgmPosition(5);
        c.ProcessKey(0);
        c.ProcessKey(KeyGKey);
        dp = c.ProcessKey(KeyAdd);
        assertEquals("!=0", 5, cs.getPrgmPosition());
    }

    @Test
    public void row4White()
    {
        DisplayPacket dp;

        // decimal mode
        c.ProcessKey(KeyDEC);
        // 16 bit word size
        c.ProcessKey(1);
        c.ProcessKey(6);
        c.ProcessKey(KeyFKey);
        c.ProcessKey(KeySTO);
        // 1's complement mode
        c.ProcessKey(KeyFKey);
        c.ProcessKey(1);

        // I think this is already adequately covered...
        c.ProcessKey(KeyEnt);
        c.ProcessKey(5);
        c.ProcessKey(KeyEnt);
        c.ProcessKey(6);
        dp = c.ProcessKey(KeyAdd);
        assertEquals("Add", "11 d", dp.getDisplayText());
    }
}

